@@ -1,4 +1,4 @@
-/*  $Id: radius.c 8469 2009-05-16 12:43:53Z iulius $
+/*  $Id: radius.c 8921 2010-01-22 23:33:08Z iulius $
 **
 **  Authenticate a user against a remote radius server.
 */
@@ -21,7 +21,7 @@
 #include "inn/md5.h"
 #include "inn/messages.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 #include "conffile.h"
 
