@@ -1,4 +1,4 @@
-/* $Id: actsync.c 8232 2008-12-14 17:05:57Z iulius $
+/* $Id: actsync.c 9023 2010-03-21 16:49:12Z iulius $
  *
  * actsync - sync or merge two active files
  *
@@ -949,7 +949,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 	/* 
 	 * check for bad chars in the hi water mark 
 	 */
-	for (p=cur->hi, i=0; *p && isascii(*p) && isdigit((int)*p); ++p, ++i) {
+	for (p=cur->hi, i=0; *p && isdigit((unsigned char) *p); ++p, ++i) {
 	}
 	if (*p) {
 	    if (!QUIET(hostid))
@@ -981,7 +981,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 	/* 
 	 * check for bad chars in the low water mark 
 	 */
-	for (p=cur->low, i=0; *p && isascii(*p) && isdigit((int)*p); ++p, ++i) {
+	for (p=cur->low, i=0; *p && isdigit((unsigned char) *p); ++p, ++i) {
 	}
 	if (*p) {
 	    if (!QUIET(hostid))
@@ -1012,16 +1012,16 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 
 	/* check for a bad group type */
 	switch (cur->type[0]) {
-	case 'y':
+	case NF_FLAG_OK:
 		/* of COURSE: collabra has incompatible flags. but it	*/
 		/* looks like they can be fixed easily enough.		*/
 		if (cur->type[1] == 'g') {
 			cur->type[1] = '\0';
 		}
-	case 'm':
-	case 'j':
-	case 'n':
-	case 'x':
+	case NF_FLAG_MODERATED:
+	case NF_FLAG_JUNK:
+	case NF_FLAG_NOLOCAL:
+	case NF_FLAG_IGNORE:
 	    if (cur->type[1] != '\0') {
 		if (!QUIET(hostid))
                     warn("line %d <%s> from %s has a bad newsgroup type",
@@ -1029,7 +1029,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 		cur->ignore |= ERROR_BADTYPE;
 	    }
 	    break;
-	case '=':
+	case NF_FLAG_ALIAS:
 	    if (cur->type[1] == '\0') {
 		if (!QUIET(hostid))
                     warn("line %d <%s> from %s has an empty =group name",
@@ -1049,7 +1049,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 	}
 
 	/* if an = type, check for bad = name */
-	if (cur->type[0] == '=' && bad_grpname(&(cur->type[1]), num_check)) {
+	if (cur->type[0] == NF_FLAG_ALIAS && bad_grpname(&(cur->type[1]), num_check)) {
 	    if (!QUIET(hostid))
                 warn("line %d <%s> from %s is equivalenced to a bad name:"
                      " <%s>", cnt+1, cur->name, host,
@@ -1059,7 +1059,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 	}
 
 	/* if an = type, check for long = name if requested */
-	if (cur->type[0] == '=' && s_flag > 0 &&
+	if (cur->type[0] == NF_FLAG_ALIAS && s_flag > 0 &&
 	    strlen(&(cur->type[1])) > (size_t)s_flag) {
 	    if (!QUIET(hostid))
                 warn("line %d <%s> from %s is equivalenced to a long name:"
@@ -1092,7 +1092,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
 	QIOclose(qp);
     } else {
 	CAclose();
-	fprintf(ToServer, "quit\r\n");
+	fprintf(ToServer, "QUIT\r\n");
 	fclose(ToServer);
 	fgets(buff, sizeof buff, FromServer);
 	fclose(FromServer);
@@ -1108,9 +1108,7 @@ get_active(char *host, int hostid, int *len, struct grp *grp, int *errs)
  *
  *	[.+-_]
  *
- * One cannot have two '.'s in a row.  The first character must be
- * alphanumeric.  The character following a '.' must be alphanumeric.
- * The name cannot end in a '.' character.
+ * One cannot have two '.'s in a row or end in a '.' character.
  *
  * If we are checking for all numeric components, (see num_chk) then
  * a component cannot be all numeric.  I.e,. there must be a non-numeric
@@ -1140,14 +1138,11 @@ bad_grpname(char *name, int num_chk)
 	return 1;
     }
 
-    /* must start with an alphanumeric ascii character */
-    if (!isascii(name[0])) {
-	return 1;
-    }
     /* set non_num as needed */
-    if (isalpha((int)name[0])) {
+    if (isalpha((unsigned char) name[0])
+        || name[0] == '+' || name[0] == '-' || name[0] == '_') {
 	non_num = true;
-    } else if ((int)isdigit((int)name[0])) {
+    } else if (isdigit((unsigned char) name[0])) {
 	non_num = false;
     } else {
 	return 1;
@@ -1156,20 +1151,14 @@ bad_grpname(char *name, int num_chk)
     /* scan each char */
     level = 0;
     for (p=name+1; *p; ++p) {
-
-	/* name must contain ASCII chars */
-	if (!isascii(*p)) {
-	    return 1;
-	}
-
 	/* alpha chars are ok */
-	if (isalpha((int)*p)) {
+	if (isalpha((unsigned char) *p)) {
 	    non_num = true;
 	    continue;
 	}
 
 	/* numeric chars are ok */
-	if (isdigit((int)*p)) {
+	if (isdigit((unsigned char) *p)) {
 	    continue;
 	}
 
@@ -1190,7 +1179,8 @@ bad_grpname(char *name, int num_chk)
 	    }
 
 	    /*
-	     * A '.' is ok as long as the next character is alphanumeric.
+             * A '.' is ok as long as the next character is alphanumeric
+             * or '+', '-', '_'.
 	     * This implies that '.' cannot be before a previous '.' and
 	     * that it cannot be at the end.
 	     *
@@ -1199,9 +1189,10 @@ bad_grpname(char *name, int num_chk)
 	     * last '.', or before the beginning if no previous '.'
 	     * has been seen.
 	     */
-	    if ((!num_chk || non_num) && isascii(*(p+1)) && isalnum((int)*(p+1))) {
+            if ((!num_chk || non_num) &&
+                (isalnum((unsigned char) *(p+1)) || *(p+1) == '+' || *(p+1) == '-' || *(p+1) == '_' )) {
 		++p;		/* '.' is ok, and so is the next char */
-		if (isdigit((int)*p)) {	/* reset non_num as needed */
+		if (isdigit((unsigned char) *p)) {	/* reset non_num as needed */
 		    non_num = false;
 		} else {
 		    non_num = true;
@@ -1345,18 +1336,18 @@ get_ignore(char *filename, int *len)
 
 	    /* ensure that this next token is a valid type */
 	    switch (p[0]) {
-	    case 'y':
-	    case 'm':
-	    case 'j':
-	    case 'n':
-	    case 'x':
+	    case NF_FLAG_OK:
+	    case NF_FLAG_MODERATED:
+	    case NF_FLAG_JUNK:
+	    case NF_FLAG_NOLOCAL:
+	    case NF_FLAG_IGNORE:
 		if (p[1] != '\0') {
                     warn("field %d on line %d of %s not a valid type",
                          i, linenum, filename);
                     die("valid types are a char from [ymnjx=] or =name");
 		}
 		break;
-	    case '=':
+	    case NF_FLAG_ALIAS:
 		break;
 	    default:
                 warn("field %d on line %d of %s is not a valid type",
@@ -1368,12 +1359,12 @@ get_ignore(char *filename, int *len)
 	    cur->type_match = 1;
 
 	    /* ensure that type is not a duplicate */
-	    if ((p[0] == 'y' && cur->y_type) ||
-	        (p[0] == 'm' && cur->m_type) ||
-	        (p[0] == 'n' && cur->n_type) ||
-	        (p[0] == 'j' && cur->j_type) ||
-	        (p[0] == 'x' && cur->x_type) ||
-	        (p[0] == '=' && cur->eq_type)) {
+	    if ((p[0] == NF_FLAG_OK && cur->y_type) ||
+	        (p[0] == NF_FLAG_MODERATED && cur->m_type) ||
+	        (p[0] == NF_FLAG_NOLOCAL && cur->n_type) ||
+	        (p[0] == NF_FLAG_JUNK && cur->j_type) ||
+	        (p[0] == NF_FLAG_IGNORE && cur->x_type) ||
+	        (p[0] == NF_FLAG_ALIAS && cur->eq_type)) {
                 warn("only one %c type allowed per line", p[0]);
                 die("field %d on line %d of %s is a duplicate type",
                     i, linenum, filename);
@@ -1381,24 +1372,24 @@ get_ignore(char *filename, int *len)
 
 	    /* note what we have seen */
 	    switch (p[0]) {
-	    case 'y':
+	    case NF_FLAG_OK:
 		cur->y_type = 1;
 		break;
-	    case 'm':
+	    case NF_FLAG_MODERATED:
 		cur->m_type = 1;
 		break;
-	    case 'j':
+	    case NF_FLAG_JUNK:
 		cur->j_type = 1;
 		break;
-	    case 'n':
+	    case NF_FLAG_NOLOCAL:
 		cur->n_type = 1;
 		break;
-	    case 'x':
+	    case NF_FLAG_IGNORE:
 		cur->x_type = 1;
 		break;
-	    case '=':
+	    case NF_FLAG_ALIAS:
 		cur->eq_type = 1;
-		if (p[0] == '=' && p[1] != '\0')
+		if (p[0] == NF_FLAG_ALIAS && p[1] != '\0')
                     cur->epat = xstrdup(p + 1);
 		break;
 	    }
@@ -1470,22 +1461,22 @@ ignore(struct grp *grp, int grplen, struct pat *igcl, int iglen)
 
 		/* specific type required, check for match */
 		switch (gp->type[0]) {
-		case 'y':
+		case NF_FLAG_OK:
 		    if (! pp->y_type) continue;  /* pattern does not apply */
 		    break;
-		case 'm':
+		case NF_FLAG_MODERATED:
 		    if (! pp->m_type) continue;  /* pattern does not apply */
 		    break;
-		case 'n':
+		case NF_FLAG_NOLOCAL:
 		    if (! pp->n_type) continue;  /* pattern does not apply */
 		    break;
-		case 'j':
+		case NF_FLAG_JUNK:
 		    if (! pp->j_type) continue;  /* pattern does not apply */
 		    break;
-		case 'x':
+		case NF_FLAG_IGNORE:
 		    if (! pp->x_type) continue;  /* pattern does not apply */
 		    break;
-		case '=':
+		case NF_FLAG_ALIAS:
 		    if (! pp->eq_type) continue;  /* pattern does not apply */
 		    if (pp->epat != NULL && !uwildmat(&gp->type[1], pp->epat)) {
 			/* equiv pattern doesn't match, patt does not apply */
@@ -2353,7 +2344,7 @@ mark_eq_probs(struct grp *grp, int grplen, int hostid, char *host1,
 	if (grp[i].hostid == hostid &&
 	    ! IS_ERROR(grp[i].ignore) &&
 	    grp[i].type != NULL &&
-	    grp[i].type[0] == '=') {
+	    grp[i].type[0] == NF_FLAG_ALIAS) {
 	    ++eq_cnt;
 	}
     }
@@ -2371,7 +2362,7 @@ mark_eq_probs(struct grp *grp, int grplen, int hostid, char *host1,
 	if (grp[i].hostid == hostid &&
 	    ! IS_ERROR(grp[i].ignore) &&
 	    grp[i].type != NULL &&
-	    grp[i].type[0] == '=') {
+	    grp[i].type[0] == NF_FLAG_ALIAS) {
 
 	    /* initialize record */
 	    eqgrp[j].skip = 0;
@@ -2436,7 +2427,7 @@ mark_eq_probs(struct grp *grp, int grplen, int hostid, char *host1,
 		}
 
 		/* if =group refers to a valid group, we are done with it */
-		if (grp[i].type != NULL && grp[i].type[0] != '=') {
+		if (grp[i].type != NULL && grp[i].type[0] != NF_FLAG_ALIAS) {
 		    eqgrp[j].skip = 1;
 		    --new_eq_cnt;
 		/* otherwise note the equiv name */
@@ -2569,7 +2560,7 @@ exec_cmd(int mode, const char *cmd, char *grp, char *type, const char *who)
 	    exit(43);
 	}
 
-	/* if non-empty line doesn't start with 'y' or 'Y', skip command */
+	/* If non-empty line doesn't start with 'y' or 'Y', skip command. */
 	if (buf[0] != 'y' && buf[0] != 'Y' && buf[0] != '\n') {
 	    /* indicate nothing was done */
 	    return 0;
