@@ -26,9 +26,10 @@ ProcessLine(char *line)
     char *tokenptr;
     int len;
     ARTHANDLE *art;
-    ARTHANDLE newart = ARTHANDLE_INITIALIZER;
+    ARTHANDLE newart;
     TOKEN token, newtoken;
     char *arttmp;
+    time_t arrived;
 
     tokenptr = line;
     
@@ -38,14 +39,11 @@ ProcessLine(char *line)
 	tokenptr[len-1] = '\0';
     }
 
-    if (!IsToken(tokenptr)) {
-        fprintf(stderr, "%s: bad token format %s\n", ME, tokenptr);
-        return;
-    }
     token = TextToToken(tokenptr);
     if ((art = SMretrieve(token, RETR_ALL)) == NULL) return;
 
     len = art->len;
+    arrived = art->arrived;
     arttmp = xmalloc(len);
     memcpy(arttmp, art->data, len);
     SMfreearticle(art);
