@@ -1,4 +1,4 @@
-/*  $Id: innconf.h 8460 2009-05-12 18:16:55Z iulius $
+/*  $Id: innconf.h 8927 2010-01-22 23:39:07Z iulius $
 **
 **  inn.conf parser interface.
 **
@@ -16,7 +16,8 @@
 **  This structure is organized in the same order as the variables contained
 **  in it are mentioned in the inn.conf documentation, and broken down into
 **  the same sections.  Note that due to the implementation, only three types
-**  of variables are permissible here:  char *, bool, and long.
+**  of variables are permissible here:  char *, bool, struc vector *, long
+**  and unsigned long.
 */
 struct innconf {
     /* General Settings */
@@ -30,20 +31,20 @@ struct innconf {
     char *runasgroup;           /* Group to run under */
 
     /* Feed Configuration */
-    long artcutoff;             /* Max accepted article age */
+    unsigned long artcutoff;    /* Max accepted article age */
     char *bindaddress;          /* Which interface IP to bind to */
     char *bindaddress6;         /* Which interface IPv6 to bind to */
     bool dontrejectfiltered;    /* Don't reject filtered article? */
-    long hiscachesize;          /* Size of the history cache in kB */
+    unsigned long hiscachesize; /* Size of the history cache in kB */
     bool ignorenewsgroups;      /* Propagate cmsgs by affected group? */
     bool immediatecancel;       /* Immediately cancel timecaf messages? */
-    long linecountfuzz;         /* Check linecount and reject if off by more */
-    long maxartsize;            /* Reject articles bigger than this */
-    long maxconnections;        /* Max number of incoming NNTP connections */
+    unsigned long linecountfuzz;/* Check linecount and reject if off by more */
+    unsigned long maxartsize;   /* Reject articles bigger than this */
+    unsigned long maxconnections; /* Max number of incoming NNTP connections */
     char *pathalias;            /* Prepended Host for the Path: line */
     char *pathcluster;          /* Appended Host for the Path: line */
     bool pgpverify;             /* Verify control messages with pgpverify? */
-    long port;                  /* Which port innd should listen on */
+    unsigned long port;         /* Which port innd should listen on */
     bool refusecybercancels;    /* Reject message IDs with "<cancel."? */
     bool remembertrash;         /* Put unwanted article IDs into history */
     char *sourceaddress;        /* Source IP for outgoing NNTP connections */
@@ -51,14 +52,14 @@ struct innconf {
     bool verifycancels;         /* Verify cancels against article author */
     bool verifygroups;          /* Check if all newsgroups exist */
     bool wanttrash;             /* Put unwanted articles in junk */
-    long wipcheck;              /* How long to defer other copies of article */
-    long wipexpire;             /* How long to keep pending article record */
+    unsigned long wipcheck;     /* How long to defer other copies of article */
+    unsigned long wipexpire;    /* How long to keep pending article record */
 
     /* History settings */
     char *hismethod;            /* Which history method to use */
     
     /* Article Storage */
-    long cnfscheckfudgesize;    /* Additional CNFS integrity checking */
+    unsigned long cnfscheckfudgesize; /* Additional CNFS integrity checking */
     bool enableoverview;        /* Store overview info for articles? */
     struct vector
       *extraoverviewadvertised; /* Extra overview fields for LIST OVERVIEW.FMT */
@@ -67,7 +68,7 @@ struct innconf {
     bool groupbaseexpiry;       /* Do expiry by newsgroup? */
     bool mergetogroups;         /* Refile articles from to.* into to */
     bool nfswriter;             /* Use NFS writer functionality */
-    long overcachesize;         /* fd size cache for tradindexed */
+    unsigned long overcachesize; /* fd size cache for tradindexed */
     char *ovgrouppat;           /* Newsgroups to store overview for */
     char *ovmethod;             /* Which overview method to use */
     bool storeonxref;           /* SMstore use Xref to detemine class? */
@@ -78,14 +79,14 @@ struct innconf {
     /* Reading */
     bool allownewnews;          /* Allow use of the NEWNEWS command */
     bool articlemmap;           /* Use mmap to read articles? */
-    long clienttimeout;         /* How long nnrpd can be inactive */
-    long initialtimeout;        /* How long nnrpd waits for first command */
-    long msgidcachesize;        /* Number of entries in the message ID cache */
+    unsigned long clienttimeout;  /* How long nnrpd can be inactive */
+    unsigned long initialtimeout; /* How long nnrpd waits for first command */
+    unsigned long msgidcachesize; /* Number of entries in the message ID cache */
     bool nfsreader;             /* Use NFS reader functionality */
-    long nfsreaderdelay;        /* Delay applied to article arrival */
+    unsigned long nfsreaderdelay; /* Delay applied to article arrival */
     bool nnrpdcheckart;         /* Check article existence before returning? */
     char *nnrpdflags;           /* Arguments to pass when spawning nnrpd */
-    long nnrpdloadlimit;        /* Maximum getloadvg() we allow */
+    unsigned long nnrpdloadlimit; /* Maximum getloadvg() we allow */
     bool noreader;              /* Refuse to fork nnrpd for readers? */
     bool readerswhenstopped;    /* Allow nnrpd when server is paused */
     bool readertrack;           /* Use the reader tracking system? */
@@ -93,9 +94,9 @@ struct innconf {
 
     /* Reading -- Keyword Support */
     bool keywords;              /* Generate keywords in overview? */
-    long keyartlimit;           /* Max article size for keyword generation */
-    long keylimit;              /* Max allocated space for keywords */
-    long keymaxwords;           /* Max count of interesting words */
+    unsigned long keyartlimit;  /* Max article size for keyword generation */
+    unsigned long keylimit;     /* Max allocated space for keywords */
+    unsigned long keymaxwords;  /* Max count of interesting words */
 
     /* Posting */
     bool addnntppostingdate;    /* Add NNTP-Posting-Date: to posts */
@@ -103,11 +104,11 @@ struct innconf {
     bool checkincludedtext;     /* Reject if too much included text */
     char *complaints;           /* Address for X-Complaints-To: */
     char *fromhost;             /* Host for the From: line */
-    long localmaxartsize;       /* Max article size of local postings */
+    unsigned long localmaxartsize; /* Max article size of local postings */
     char *moderatormailer;      /* Default host to mail moderated articles */
     bool nnrpdauthsender;       /* Add authenticated Sender: header? */
     char *nnrpdposthost;        /* Host postings should be forwarded to */
-    long nnrpdpostport;         /* Port postings should be forwarded to */
+    unsigned long nnrpdpostport; /* Port postings should be forwarded to */
     char *organization;         /* Data for the Organization: header */
     bool spoolfirst;            /* Spool all posted articles? */
     bool strippostcc;           /* Strip To:, Cc: and Bcc: from posts */
@@ -115,10 +116,10 @@ struct innconf {
     /* Posting -- Exponential Backoff */
     bool backoffauth;           /* Backoff by user, not IP address */
     char *backoffdb;            /* Directory for backoff databases */
-    long backoffk;              /* Multiple for the sleep time */
-    long backoffpostfast;       /* Upper time limit for fast posting */
-    long backoffpostslow;       /* Lower time limit for slow posting */
-    long backofftrigger;        /* Number of postings before triggered */
+    unsigned long backoffk;     /* Multiple for the sleep time */
+    unsigned long backoffpostfast; /* Upper time limit for fast posting */
+    unsigned long backoffpostslow; /* Lower time limit for slow posting */
+    unsigned long backofftrigger;  /* Number of postings before triggered */
 
     /* Reading and posting -- SSL and TLS support */
 #ifdef HAVE_SSL
@@ -130,45 +131,46 @@ struct innconf {
 
     /* Monitoring */
     bool doinnwatch;            /* Start innwatch from rc.news? */
-    long innwatchbatchspace;    /* Minimum free space in pathoutgoing */
-    long innwatchlibspace;      /* Minimum free space in pathdb */
-    long innwatchloload;        /* Load times 100 at which to restart */
-    long innwatchhiload;        /* Load times 100 at which to throttle */
-    long innwatchpauseload;     /* Load times 100 at which to pause */
-    long innwatchsleeptime;     /* Seconds to wait between checks */
-    long innwatchspoolnodes;    /* Minimum free inodes in patharticles */
-    long innwatchspoolspace;    /* Minimum free space in patharticles */
+    unsigned long innwatchbatchspace; /* Minimum free space in pathoutgoing */
+    unsigned long innwatchlibspace; /* Minimum free space in pathdb */
+    unsigned long innwatchloload; /* Load times 100 at which to restart */
+    unsigned long innwatchhiload; /* Load times 100 at which to throttle */
+    unsigned long innwatchpauseload; /* Load times 100 at which to pause */
+    unsigned long innwatchsleeptime; /* Seconds to wait between checks */
+    unsigned long innwatchspoolnodes; /* Minimum free inodes in patharticles */
+    unsigned long innwatchspoolspace; /* Minimum free space in patharticles */
 
     /* Logging */
     bool docnfsstat;            /* Run cnfsstat in the background? */
-    long incominglogfrequency;  /* Checkpoint log after this many articles */
+    unsigned long incominglogfrequency;  /* Checkpoint log after this many articles */
     bool logartsize;            /* Log article sizes? */
     bool logcancelcomm;         /* Log ctlinnd cancel commands to syslog? */
-    long logcycles;             /* How many old logs scanlogs should keep */
+    unsigned long logcycles;    /* How many old logs scanlogs should keep */
     bool logipaddr;             /* Log by host IP address? */
     bool logsitename;           /* Log outgoing site names? */
     bool logstatus;             /* Send a status report to syslog? */
+    bool logtrash;              /* Log unwanted newsgroups? */
     bool nnrpdoverstats;        /* Log overview statistics? */
     bool nntplinklog;           /* Put storage token into the log? */
-    long status;                /* Status file update interval */
+    unsigned long status;       /* Status file update interval */
     char *stathist;             /* Filename for history profiler outputs */
-    long timer;                 /* Performance monitoring interval */
+    unsigned long timer;        /* Performance monitoring interval */
 
     /* System Tuning */
-    long badiocount;            /* Failure count before dropping channel */
-    long blockbackoff;          /* Multiplier for sleep in EAGAIN writes */
-    long chaninacttime;         /* Wait before noticing inactive channels */
-    long chanretrytime;         /* How long before channel restarts */
-    long datamovethreshold;     /* Threshold to extend buffer or move data */
-    long icdsynccount;          /* Articles between active & history updates */
-    long keepmmappedthreshold;  /* Threshold for keeping mmap in buffindexed */
-    long maxcmdreadsize;        /* Max NNTP command read size used by innd */
-    long maxforks;              /* Give up after this many fork failure */
+    unsigned long badiocount;   /* Failure count before dropping channel */
+    unsigned long blockbackoff; /* Multiplier for sleep in EAGAIN writes */
+    unsigned long chaninacttime;/* Wait before noticing inactive channels */
+    unsigned long chanretrytime;/* How long before channel restarts */
+    unsigned long datamovethreshold; /* Threshold to extend buffer or move data */
+    unsigned long icdsynccount; /* Articles between active & history updates */
+    unsigned long keepmmappedthreshold; /* Threshold for keeping mmap in buffindexed */
+    unsigned long maxcmdreadsize; /* Max NNTP command read size used by innd */
+    unsigned long maxforks;     /* Give up after this many fork failure. */
     long nicekids;              /* Child processes get niced to this */
-    long nicenewnews;           /* If NEWNEWS command is used, nice to this */
-    long nicennrpd;             /* nnrpd is niced to this */
-    long pauseretrytime;        /* Seconds before seeing if pause is ended */
-    long peertimeout;           /* How long peers can be inactive */
+    unsigned long nicenewnews;  /* If NEWNEWS command is used, nice to this */
+    unsigned long nicennrpd;    /* nnrpd is niced to this */
+    unsigned long pauseretrytime; /* Seconds before seeing if pause is ended */
+    unsigned long peertimeout;  /* How long peers can be inactive */
     long rlimitnofile;          /* File descriptor limit to set */
 
     /* Paths */
