@@ -1,4 +1,4 @@
-/*  $Id: art.c 8657 2009-10-10 07:58:53Z iulius $
+/*  $Id: art.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Article-processing.
 */
@@ -68,18 +68,12 @@ static ARTOVERFIELD	*ARTfields;
 */
 static char	ARTctl[] = "control";
 static char	ARTjnk[] = "junk";
-static char	*ARTpathme;
 
 /*
-**  Flag array, indexed by character.  Character classes for Message-ID's.
+**  Flag array, indexed by character.  Character classes for hostnames.
 */
-static char		ARTcclass[256];
-#define CC_MSGID_ATOM	01
-#define CC_MSGID_NORM	02
-#define CC_HOSTNAME	04
-#define ARTnormchar(c)	((ARTcclass[(unsigned char)(c)] & CC_MSGID_NORM) != 0)
-#define ARTatomchar(c)	((ARTcclass[(unsigned char)(c)] & CC_MSGID_ATOM) != 0)
-#define ARThostchar(c)	((ARTcclass[(unsigned char)(c)] & CC_HOSTNAME) != 0)
+static char             hostcclass[256];
+#define ARThostchar(c)  ((hostcclass[(unsigned char)(c)]) != 0)
 
 #if defined(DO_PERL) || defined(DO_PYTHON)
 const char	*filterPath;
@@ -239,31 +233,27 @@ ARTcompare(const void *p1, const void *p2)
 void
 ARTsetup(void)
 {
-  const char *	p;
   const ARTHEADER **	table;
+  const unsigned char *p;
   unsigned int	i;
 
   /* Set up the character class tables.  These are written a
    * little strangely to work around a GCC2.0 bug. */
-  memset(ARTcclass, 0, sizeof ARTcclass);
-  p = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+  memset(hostcclass, 0, sizeof(hostcclass));
+
+  p = (const unsigned char*) "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
   while ((i = *p++) != 0) {
-    ARTcclass[i] = CC_HOSTNAME | CC_MSGID_ATOM | CC_MSGID_NORM;
-  }
-  p = "!#$%&'*+-/=?^_`{|}~";
-  while ((i = *p++) != 0) {
-    ARTcclass[i] = CC_MSGID_ATOM | CC_MSGID_NORM;
-  }
-  p = "\"(),.:;<@[\\]";
-  while ((i = *p++) != 0) {
-    ARTcclass[i] = CC_MSGID_NORM;
+    hostcclass[i] = 1;
   }
 
-  /* The RFC's don't require it, but we add underscore to the list of valid
-   * hostname characters. */
-  ARTcclass['.'] |= CC_HOSTNAME;
-  ARTcclass['-'] |= CC_HOSTNAME;
-  ARTcclass['_'] |= CC_HOSTNAME;
+  /* The RFCs don't require it, but we add underscore to the list
+   * of valid hostname characters. */
+  hostcclass['.'] = 1;
+  hostcclass['-'] = 1;
+  hostcclass['_'] = 1;
+
+  /* Also initialize the character class tables for message-IDs. */
+  InitializeMessageIDcclass();
 
   /* Build the header tree. */
   table = xmalloc(ARRAY_SIZE(ARTheaders) * sizeof(ARTHEADER *));
@@ -273,10 +263,6 @@ ARTsetup(void)
   ARTheadertree = ARTbuildtree(table, 0, ARRAY_SIZE(ARTheaders));
   free(table);
 
-  /* Get our Path name, kill trailing !. */
-  ARTpathme = xstrdup(Path.data);
-  ARTpathme[Path.used - 1] = '\0';
-
   /* Set up database; ignore errors. */
   ARTreadschema();
 }
@@ -395,8 +381,10 @@ ARTlogreject(CHANNEL *cp, const char *text)
     char **hops;
 
     /* We may still haven't received the message-ID of the rejected article. */
-    if (HDR_FOUND(HDR__MESSAGE_ID))
+    if (HDR_FOUND(HDR__MESSAGE_ID)) {
+        HDR_LASTCHAR_SAVE(HDR__MESSAGE_ID);
         HDR_PARSE_START(HDR__MESSAGE_ID);
+    }
 
     /* Set up the headers that we want to use.  We only need to parse the path
        on rejections if logipaddr is false or we can't find a good host. */
@@ -408,6 +396,7 @@ ARTlogreject(CHANNEL *cp, const char *text)
         }
     } else {
         if (HDR_FOUND(HDR__PATH)) {
+            HDR_LASTCHAR_SAVE(HDR__PATH);
             HDR_PARSE_START(HDR__PATH);
             hopcount =
                 ARTparsepath(HDR(HDR__PATH), HDR_LEN(HDR__PATH), &data->Path);
@@ -660,7 +649,7 @@ ARTcheckheader(CHANNEL *cp, int size)
   }
 
   /* See if this is a system header.  A fairly tightly-coded binary search. */
-  c = CTYPE(islower, *header) ? toupper(*header) : *header;
+  c = islower((unsigned char) *header) ? toupper(*header) : *header;
   for (*colon = '\0', tp = ARTheadertree; tp; ) {
     if ((i = c - tp->Name[0]) == 0 && (i = strcasecmp(header, tp->Name)) == 0)
       break;
@@ -698,7 +687,8 @@ ARTcheckheader(CHANNEL *cp, int size)
     hc->Length = -1;
   } else {
     /* We need to remove leading and trailing spaces for
-     * message-IDs; otherwise, ARTidok() will fail. */
+     * message-IDs; otherwise, history hashes may not be
+     * correctly computed. */
     if (i == HDR__MESSAGE_ID || i == HDR__SUPERSEDES) {
       for (p = colon + 1 ; (p < header + size - 2) &&
            (ISWHITE(*p)) ; p++);
@@ -715,86 +705,6 @@ ARTcheckheader(CHANNEL *cp, int size)
   return;
 }
 
-/*
-**  Check Message-ID format based on RFC 822 grammar, except that (as per
-**  RFC 1036) whitespace, non-printing, and '>' characters are excluded.
-**  Based on code by Paul Eggert posted to news.software.b on 22-Nov-90
-**  in <#*tyo2'~n@twinsun.com>, with additional e-mail discussion.
-**  Thanks, Paul.
-*/
-bool
-ARTidok(const char *MessageID)
-{
-  int		c;
-  const char	*p;
-
-  /* Check the length of the message ID. */
-  if (MessageID == NULL || strlen(MessageID) > NNTP_MAXLEN_MSGID)
-    return false;
-
-  /* Scan local-part:  "< atom|quoted [ . atom|quoted]" */
-  p = MessageID;
-  if (*p++ != '<')
-    return false;
-  for (; ; p++) {
-    if (ARTatomchar(*p))
-      while (ARTatomchar(*++p))
-	continue;
-    else {
-      if (*p++ != '"')
-	return false;
-      for ( ; ; ) {
-	switch (c = *p++) {
-	case '\\':
-	  c = *p++;
-	  /* FALLTHROUGH */
-	default:
-	  if (ARTnormchar(c))
-	    continue;
-	  return false;
-	case '"':
-	  break;
-	}
-	break;
-      }
-    }
-    if (*p != '.')
-      break;
-  }
-
-  /* Scan domain part:  "@ atom|domain [ . atom|domain] > \0" */
-  if (*p++ != '@')
-    return false;
-  for ( ; ; p++) {
-    if (ARTatomchar(*p))
-      while (ARTatomchar(*++p))
-	continue;
-    else {
-      if (*p++ != '[')
-	return false;
-      for ( ; ; ) {
-	switch (c = *p++) {
-	case '\\':
-	  c = *p++;
-	  /* FALLTHROUGH */
-	default:
-	  if (ARTnormchar(c))
-	    continue;
-	  /* FALLTHROUGH */
-	case '[':
-	  return false;
-	case ']':
-	  break;
-	}
-	break;
-      }
-    }
-    if (*p != '.')
-      break;
-  }
-
-  return *p == '>' && *++p == '\0';
-}
 
 /*
 **  Clean up data field where article informations are stored.
@@ -810,6 +720,7 @@ ARTprepare(CHANNEL *cp)
   for (i = 0 ; i < MAX_ARTHEADER ; i++, hc++) {
     hc->Value = NULL;
     hc->Length = 0;
+    hc->LastChar = '\r';
   }
   data->Lines = data->HeaderLines = data->CRwithoutLF = data->LFwithoutCR = 0;
   data->DotStuffedLines = 0;
@@ -853,26 +764,28 @@ ARTchecksize(CHANNEL *cp)
     const char *msgid;
 
     size = cp->Next - cp->Start;
-    if (innconf->maxartsize > 0 && size > (size_t) innconf->maxartsize) {
+    if (innconf->maxartsize != 0 && size > innconf->maxartsize) {
         if (cp->State == CSgotarticle || cp->State == CSgotlargearticle)
             cp->State = CSgotlargearticle;
         else
             cp->State = CSeatarticle;
     }
     if (cp->State == CSgotlargearticle) {
-        notice("%s internal rejecting huge article (%lu > %ld)", CHANname(cp),
+        notice("%s internal rejecting huge article (%lu > %lu)", CHANname(cp),
                (unsigned long) size, innconf->maxartsize);
-        ARTerror(cp, "Article of %lu bytes exceeds local limit of %ld bytes",
+        ARTerror(cp, "Article of %lu bytes exceeds local limit of %lu bytes",
                  (unsigned long) size, innconf->maxartsize);
 
 	/* Write a local cancel entry so nobody else gives it to us. */
 	if (HDR_FOUND(HDR__MESSAGE_ID)) {
+            HDR_LASTCHAR_SAVE(HDR__MESSAGE_ID);
             HDR_PARSE_START(HDR__MESSAGE_ID);
             msgid = HDR(HDR__MESSAGE_ID);
             /* The article posting time has not been parsed.  We cannot
              * give it to InndHisRemember. */
             if (!HIScheck(History, msgid) && !InndHisRemember(msgid, 0))
                 warn("SERVER cant write %s", msgid);
+            HDR_PARSE_END(HDR__MESSAGE_ID);
 	}
     }
 }
@@ -903,14 +816,24 @@ ARTparseheader(CHANNEL *cp)
            terminator (five characters).  If we don't have at least five more
            characters, we're guaranteed that the article isn't complete, so
            save ourselves complexity and just return and wait for more
-           data. */
+           data.
+           We also have to check whether the headers begin with the article
+           terminator. */
+        if ((i == cp->Start + 1) && (bp->used - i > 1)
+            && (memcmp(&bp->data[i - 1], ".\r\n", 3) == 0)) {
+            ARTerror(cp, "Empty article");
+            cp->State = CSnoarticle;
+            cp->Next = i + 2;
+            return;
+        }
+
         if (bp->used - i < 5) {
             cp->Next = i;
             return;
         }
         if (memcmp(&bp->data[i], "\r\n.\r\n", 5) == 0) {
             if (i == cp->Start) {
-                ARTerror(cp, "Empty article");
+                ARTerror(cp, "Empty headers and body");
                 cp->State = CSnoarticle;
             } else {
                 ARTerror(cp, "No body");
@@ -1055,8 +978,10 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
   /* replace trailing '\r\n' with '\0\n' of all system header to be handled
      easily by str*() functions */
   for (i = 0 ; i < MAX_ARTHEADER ; i++) {
-    if (HDR_FOUND(i))
+    if (HDR_FOUND(i)) {
+      HDR_LASTCHAR_SAVE(i);
       HDR_PARSE_START(i);
+    }
   }
 
   /* Make sure all the headers we need are there */
@@ -1064,7 +989,7 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
     if (hp[i].Type == HTreq) {
       if (HDR_FOUND(i))
         continue;
-      if (hc[i].Length < 0) {
+      if (HDR_LEN(i) < 0) {
         sprintf(buff, "%d Duplicate \"%s\" header",
                 ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
                 hp[i].Name);
@@ -1078,8 +1003,8 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
     }
   }
 
-  /* assumes Message-ID header is required header */
-  if (!ARTidok(HDR(HDR__MESSAGE_ID))) {
+  /* Assumes the Message-ID: header is a required header. */
+  if (!IsValidMessageID(HDR(HDR__MESSAGE_ID), true)) {
     HDR_LEN(HDR__MESSAGE_ID) = 0;
     sprintf(buff, "%d Bad \"Message-ID\" header",
             ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT);
@@ -1087,11 +1012,12 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
     return false;
   }
 
-  if (innconf->linecountfuzz && HDR_FOUND(HDR__LINES)) {
+  if (innconf->linecountfuzz != 0 && HDR_FOUND(HDR__LINES)) {
     p = HDR(HDR__LINES);
     i = data->Lines;
-    if ((delta = i - atoi(p)) != 0 && abs(delta) > innconf->linecountfuzz) {
-      sprintf(buff, "%d Linecount %s != %d +- %ld",
+    if ((delta = i - atoi(p)) != 0
+        && (unsigned long) abs(delta) > innconf->linecountfuzz) {
+      sprintf(buff, "%d Linecount %s != %d +- %lu",
               ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
 	MaxLength(p, p), i, innconf->linecountfuzz);
       TMRstop(TMR_ARTCLEAN);
@@ -1102,7 +1028,7 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
   /* Is article too old? */
   /* assumes Date header is required header */
   p = HDR(HDR__DATE);
-  data->Posted = parsedate_rfc2822_lax(p);
+  data->Posted = parsedate_rfc5322_lax(p);
   if (data->Posted == (time_t) -1) {
     sprintf(buff, "%d Bad \"Date\" header -- \"%s\"",
             ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
@@ -1110,7 +1036,7 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
     TMRstop(TMR_ARTCLEAN);
     return false;
   }
-  if (innconf->artcutoff) {
+  if (innconf->artcutoff != 0) {
       long cutoff = innconf->artcutoff * 24 * 60 * 60;
 
       if (data->Posted < Now.tv_sec - cutoff) {
@@ -1130,13 +1056,13 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
   }
   if (HDR_FOUND(HDR__EXPIRES)) {
     p = HDR(HDR__EXPIRES);
-    data->Expires = parsedate_rfc2822_lax(p);
+    data->Expires = parsedate_rfc5322_lax(p);
     if (data->Expires == (time_t) -1)
       data->Expires = 0;
   }
 
-  /* Colon or whitespace in the Newsgroups header? */
-  /* assumes Newsgroups header is required header */
+  /* Colon or whitespace in the Newsgroups: header? */
+  /* Assumes Newsgroups: header is required header. */
   if ((data->Groupcount =
     NGsplit(HDR(HDR__NEWSGROUPS), HDR_LEN(HDR__NEWSGROUPS),
     &data->Newsgroups)) == 0) {
@@ -1223,7 +1149,7 @@ ARTcancelverify(const ARTDATA *data, const char *MessageID, TOKEN *token)
 
   /* Copy Newsgroups: from article be to cancelled to q.
    * Double-terminate q (sentinel). */
-  local = wire_findheader(art->data, art->len, "Newsgroups");
+  local = wire_findheader(art->data, art->len, "Newsgroups", true);
   if (local == NULL) {
     SMfreearticle(art);
     return false;
@@ -1286,9 +1212,9 @@ ARTcancel(const ARTDATA *data, const char *MessageID, const bool Trusted)
     return;
   }
 
-  if (!ARTidok(MessageID)) {
+  if (!IsValidMessageID(MessageID, true)) {
     syslog(L_NOTICE, "%s bad cancel Message-ID %s", data->Feedsite,
-      MaxLength(MessageID, MessageID));
+           MaxLength(MessageID, MessageID));
     TMRstop(TMR_ARTCNCL);
     return;
   }
@@ -1321,7 +1247,7 @@ ARTcancel(const ARTDATA *data, const char *MessageID, const bool Trusted)
     OVcancel(token);
   if (!SMcancel(token) && SMerrno != SMERR_NOENT && SMerrno != SMERR_UNINIT)
     syslog(L_ERROR, "%s cant cancel %s (SMerrno %d)", LogName,
-	TokenToText(token), SMerrno);
+           TokenToText(token), SMerrno);
   if (innconf->immediatecancel && !SMflushcacheddata(SM_CANCELLEDART))
     syslog(L_ERROR, "%s cant cancel cached %s", LogName, TokenToText(token));
   snprintf(buff, sizeof(buff), "Cancelling %s",
@@ -1342,10 +1268,10 @@ ARTcontrol(ARTDATA *data, char *Control, CHANNEL *cp UNUSED)
 
   /* See if it's a cancel message. */
   c = *Control;
-  if (c == 'c' && strncmp(Control, "cancel", 6) == 0) {
+  if ((c == 'c' || c == 'C') && strncasecmp(Control, "cancel", 6) == 0) {
     for (p = &Control[6]; ISWHITE(*p); p++)
       continue;
-    if (*p && ARTidok(p))
+    if (*p && IsValidMessageID(p, true))
       ARTcancel(data, p, false);
     return;
   }
@@ -1407,9 +1333,9 @@ DISTwanted(char **list, char *p)
   for (sawbang = false, c = *p; (q = *list) != NULL; list++) {
     if (*q == '!') {
       sawbang = true;
-      if (c == *++q && strcmp(p, q) == 0)
+      if (c == *++q && strcasecmp(p, q) == 0)
 	return false;
-    } else if (c == *q && strcmp(p, q) == 0)
+    } else if (c == *q && strcasecmp(p, q) == 0)
       return true;
   }
 
@@ -1436,17 +1362,17 @@ DISTwantany(char **site, char **article)
 }
 
 /*
-**  Send the current article to all sites that would get it if the
-**  group were created.
+**  Send the current article to all sites that would get it (used
+**  for instance to propagate newgroup control articles or junk articles).
 */
 static void
-ARTsendthegroup(char *name)
+ARTsendthegroup(char *name, char *fromgroup)
 {
   SITE		*sp;
   int		i;
   NEWSGROUP	*ngp;
 
-  for (ngp = NGfind(ARTctl), sp = Sites, i = nSites; --i >= 0; sp++) {
+  for (ngp = NGfind(fromgroup), sp = Sites, i = nSites; --i >= 0; sp++) {
     if (sp->Name != NULL && SITEwantsgroup(sp, name)) {
       SITEmark(sp, ngp);
     }
@@ -1714,7 +1640,7 @@ ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
   HDRCONTENT	*hc = data->HdrContent;
   SITE		*sp, *funnel;
   int		i, j, Groupcount, Followcount, Crosscount;
-  char	        *p, *q;
+  char	        *p, *q, *begin, savec;
   struct buffer	*bp;
   bool		sendit;
 
@@ -1731,10 +1657,37 @@ ARTpropagate(ARTDATA *data, const char **hops, int hopcount, char **list,
     sp->Sendit = false;
 	
     if (sp->Originator) {
-      if (!HDR_FOUND(HDR__XTRACE)) {
+      if (!HDR_FOUND(HDR__XTRACE) && !HDR_FOUND(HDR__INJECTION_INFO)) {
 	if (!sp->FeedwithoutOriginator)
 	  continue;
-      } else {
+      } else if (HDR_FOUND(HDR__INJECTION_INFO)) {
+        begin = (char *) skip_cfws(HDR(HDR__INJECTION_INFO));
+
+        if (begin == '\0')
+          continue;
+
+        /* The path identity ends with ';' or CFWS. */
+        for (p = begin; *p != ';' && *p != ' ' && *p != '\t' && *p != '\r'
+                        && *p != '\0'; p++) ;
+        savec = *p;
+        *p = '\0';
+
+        for (j = 0, sendit = false; (q = sp->Originator[j]) != NULL; j++) {
+          if (*q == '@') {
+            if (uwildmat(begin, &q[1])) {
+              *p = savec;
+              sendit = false;
+              break;
+            }
+          } else {
+            if (uwildmat(begin, q))
+              sendit = true;
+          }
+        }
+        *p = savec;
+        if (!sendit)
+          continue;
+      } else if (HDR_FOUND(HDR__XTRACE)) {
 	if ((p = strchr(HDR(HDR__XTRACE), ' ')) != NULL) {
 	  *p = '\0';
 	  for (j = 0, sendit = false; (q = sp->Originator[j]) != NULL; j++) {
@@ -1858,8 +1811,7 @@ ARTmakeoverview(CHANNEL *cp)
   const ARTHEADER *hp;
   char		*p, *q;
   int		i, j, len;
-  char		*key_old_value = NULL;
-  int		key_old_length = 0;
+  bool          keywords_generated = false;
 
   if (ARTfields == NULL) {
     /* User error. */
@@ -1882,11 +1834,17 @@ ARTmakeoverview(CHANNEL *cp)
        it into overview. */
     if (DO_KEYWORDS && innconf->keywords) {
       /* Ensure that there are Keywords: to shovel. */
-      if (hp == &ARTheaders[HDR__KEYWORDS]) {
-	key_old_value  = HDR(HDR__KEYWORDS);
-	key_old_length = HDR_LEN(HDR__KEYWORDS);
-	KEYgenerate(&hc[HDR__KEYWORDS], cp->In.data + data->Body,
-                    cp->Next - data->Body, key_old_value, key_old_length);
+      if (hp == &ARTheaders[HDR__KEYWORDS] && HDR(HDR__KEYWORDS) == NULL) {
+        keywords_generated = true;
+        KEYgenerate(&hc[HDR__KEYWORDS], cp->In.data + data->Body,
+                    cp->Next - data->Body);
+        /* Do not memorize an empty Keywords: header. */
+        if (HDR_LEN(HDR__KEYWORDS) == 0) {
+          if (HDR(HDR__KEYWORDS) != NULL)
+              free(HDR(HDR__KEYWORDS)); /* malloc'd within. */
+          HDR(HDR__KEYWORDS) = NULL;
+          keywords_generated = false;
+        }
       }
     }
 
@@ -1938,12 +1896,12 @@ ARTmakeoverview(CHANNEL *cp)
 
     /* Patch the old keywords back in. */
     if (DO_KEYWORDS && innconf->keywords) {
-      if (key_old_value) {
-        if (hc[HDR__KEYWORDS].Value)
-          free(hc[HDR__KEYWORDS].Value); /* malloc'd within. */
-        hc[HDR__KEYWORDS].Value  = key_old_value;
-        hc[HDR__KEYWORDS].Length = key_old_length;
-        key_old_value = NULL;
+      if (keywords_generated) {
+        if (HDR(HDR__KEYWORDS) != NULL)
+          free(HDR(HDR__KEYWORDS)); /* malloc'd within. */
+        HDR(HDR__KEYWORDS) = NULL;
+        HDR_LEN(HDR__KEYWORDS) = 0;
+        keywords_generated = false;
       }
     }
   }
@@ -1961,6 +1919,7 @@ ARTpost(CHANNEL *cp)
   int		i, j, *isp, hopcount, oerrno, canpost;
   size_t        n;
   NEWSGROUP	*ngp, **ngptr;
+  NEWSGROUP     *ngpjunk;
   SITE		*sp;
   ARTDATA	*data = &cp->Data;
   HDRCONTENT	*hc = data->HdrContent;
@@ -2053,13 +2012,13 @@ ARTpost(CHANNEL *cp)
 
   n = strlen(hops[0]);
   if (n == Path.used - 1 &&
-    strncmp(Path.data, hops[0], Path.used - 1) == 0)
+    strncasecmp(Path.data, hops[0], Path.used - 1) == 0)
     data->Hassamepath = true;
   else
     data->Hassamepath = false;
   if (Pathcluster.data != NULL &&
     n == Pathcluster.used - 1 &&
-    strncmp(Pathcluster.data, hops[0], Pathcluster.used - 1) == 0)
+    strncasecmp(Pathcluster.data, hops[0], Pathcluster.used - 1) == 0)
     data->Hassamecluster = true;
   else
     data->Hassamecluster = false;
@@ -2195,16 +2154,30 @@ ARTpost(CHANNEL *cp)
   }
 
   if (HDR_FOUND(HDR__FOLLOWUPTO)) {
-    for (i = 0, p = HDR(HDR__FOLLOWUPTO) ; (p = strchr(p, ',')) != NULL ;
-      i++, p++)
-      continue;
+    /* Count the number of commas without syntactically parsing the header. */
+    for (i = 1, p = HDR(HDR__FOLLOWUPTO) ; (p = strchr(p, ',')) != NULL ;
+         i++, p++) ;
+
     data->Followcount = i;
-  }
-  if (data->Followcount == 0)
+
+    /* When "poster" is the only value, then it is not a followup. */
+    if (i == 1) {
+      /* Skip leading whitespaces. */
+      p = (char *) skip_fws(HDR(HDR__FOLLOWUPTO));
+
+      /* Check for an empty header field or "poster". */
+      if (*p == '\0' || (strncasecmp(p, "poster", 6) == 0
+          && (p[6] == ' ' || p[6] == '\t' || p[6] == '\0'
+              || p[6] == '\r' || p[6] == '\n'))) {
+        data->Followcount = 0;
+      }
+    }
+  } else {
     data->Followcount = data->Groupcount;
+  }
 
   groups = data->Newsgroups.List;
-  /* Parse the Control header. */
+  /* Parse the Control: header. */
   LikeNewgroup = false;
   if (HDR_FOUND(HDR__CONTROL)) {
     IsControl = true;
@@ -2212,11 +2185,11 @@ ARTpost(CHANNEL *cp)
     /* Nip off the first word into lowercase. */
     strlcpy(ControlWord, HDR(HDR__CONTROL), sizeof(ControlWord));
     for (p = ControlWord; *p && !ISWHITE(*p); p++)
-      if (CTYPE(isupper, *p))
+      if (isupper((unsigned char) *p))
 	*p = tolower(*p);
     *p = '\0';
-    LikeNewgroup = (strcmp(ControlWord, "newgroup") == 0
-                    || strcmp(ControlWord, "rmgroup") == 0);
+    LikeNewgroup = (strcasecmp(ControlWord, "newgroup") == 0
+                    || strcasecmp(ControlWord, "rmgroup") == 0);
 
     if (innconf->ignorenewsgroups && LikeNewgroup) {
       for (p++; *p && ISWHITE(*p); p++);
@@ -2241,7 +2214,7 @@ ARTpost(CHANNEL *cp)
 	data->Followcount = data->Groupcount;
     }
     
-    LikeNewgroup = (LikeNewgroup || strcmp(ControlWord, "checkgroups") == 0);
+    LikeNewgroup = (LikeNewgroup || strcasecmp(ControlWord, "checkgroups") == 0);
     
     /* Control messages to "foo.ctl" are treated as if they were
      * posted to "foo".  I should probably apologize for all the
@@ -2269,8 +2242,17 @@ ARTpost(CHANNEL *cp)
          * removed (or to the admin group to which the checkgroups is posted),
          * and send it to all sites that would or would have had the group
          * if it were created. */
-        ARTsendthegroup(*groups);
+        ARTsendthegroup(*groups, ARTctl);
         Accepted = true;
+      } else if (innconf->wanttrash && !innconf->verifygroups) {
+        /* Don't set Accepted in this case, because we may still end
+         * up filing the article in the junk group. */
+        for (ngpjunk = NGfind(ARTjnk), sp = Sites, i = nSites; --i >= 0; sp++) {
+            if (sp->Name != NULL && sp->FeedTrash && SITEwantsgroup(sp, *groups)) {
+                SITEmark(sp, ngpjunk);
+            }
+        }
+        NonExist = true;
       } else {
         NonExist = true;
         /* Check if all the newsgroups listed in the Newsgroups: header of
@@ -2364,8 +2346,8 @@ ARTpost(CHANNEL *cp)
       }
     }
 
-    /* If it's excluded, don't file it. */
-    if (ngp->Rest[0] == NF_FLAG_EXCLUDED)
+    /* If it's junk, don't file it. */
+    if (ngp->Rest[0] == NF_FLAG_JUNK)
       continue;
 
     /* Expand aliases, mark the article as getting filed in the group. */
@@ -2375,13 +2357,6 @@ ARTpost(CHANNEL *cp)
     ngp->PostCount = 0;
   }
 
-  /* Loop over sites to find Poisons/ControlOnly and undo Sendit flags. */
-  for (i = nSites, sp = Sites; --i >= 0; sp++) {
-    if (sp->Poison || (sp->ControlOnly && !IsControl)
-      || (sp->DontWantNonExist && NonExist))
-      sp->Sendit = false;		
-  }
-
   /* Control messages not filed in "to" get filed only in control.name
    * or control. */
   if (IsControl && Accepted && !ToGroup) {
@@ -2407,7 +2382,7 @@ ARTpost(CHANNEL *cp)
 
   /* If !Accepted, then none of the article's newgroups exist in our
    * active file.  Proper action is to drop the article on the floor.
-   * If ngp == GroupPointers, then all the new articles newsgroups are
+   * If ngptr == GroupPointers, then all the new articles newsgroups are
    * "j" entries in the active file.  In that case, we have to file it
    * under junk so that downstream feeds can get it. */
   if (!Accepted || ngptr == GroupPointers) {
@@ -2424,7 +2399,12 @@ ARTpost(CHANNEL *cp)
                  MaxLength(data->Newsgroups.List[0],
                            data->Newsgroups.List[0]));
       }
-      ARTlog(data, ART_REJECT, cp->Error);
+
+      if (innconf->logtrash) {
+          /* Log the previous reject. */
+          ARTlog(data, ART_REJECT, cp->Error);
+      }
+
       if (!innconf->wanttrash) {
 	if (innconf->remembertrash && (Mode == OMrunning) &&
 	  !NoHistoryUpdate && !InndHisRemember(HDR(HDR__MESSAGE_ID),
@@ -2465,16 +2445,23 @@ ARTpost(CHANNEL *cp)
   }
   *ngptr = NULL;
 
+  /* Loop over sites to find Poisons/ControlOnly and undo Sendit flags. */
+  for (i = nSites, sp = Sites; --i >= 0; sp++) {
+    if (sp->Poison || (sp->ControlOnly && !IsControl)
+        || (sp->DontWantNonExist && NonExist))
+      sp->Sendit = false;
+  }
+
   if (innconf->xrefslave) {
     if (ARTxrefslave(data) == false) {
       if (HDR_FOUND(HDR__XREF)) {
 	snprintf(cp->Error, sizeof(cp->Error),
-                 "%d Xref header \"%s\" invalid in xrefslave mode",
+                 "%d Xref: header \"%s\" invalid in xrefslave mode",
                  ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
                  MaxLength(HDR(HDR__XREF), HDR(HDR__XREF)));
       } else {
 	snprintf(cp->Error, sizeof(cp->Error),
-                 "%d Xref header required in xrefslave mode",
+                 "%d Xref: header required in xrefslave mode",
                  ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT);
       }
       ARTlog(data, ART_REJECT, cp->Error);
@@ -2499,10 +2486,12 @@ ARTpost(CHANNEL *cp)
     ngp->PostCount = 0;
 
   token = ARTstore(cp);
-  /* change trailing '\r\n' to '\0\n' of all system header */
+  /* Change trailing '\r\n' to '\0\n' of all system header. */
   for (i = 0 ; i < MAX_ARTHEADER ; i++) {
-    if (HDR_FOUND(i))
+    if (HDR_FOUND(i)) {
+      HDR_LASTCHAR_SAVE(i);
       HDR_PARSE_START(i);
+    }
   }
   if (token.type == TOKEN_EMPTY) {
     syslog(L_ERROR, "%s cant store article: %s", LogName, SMerrorstr);
@@ -2611,7 +2600,7 @@ ARTpost(CHANNEL *cp)
       ARTcontrol(data, HDR(HDR__CONTROL), cp);
     }
     if (DoCancels && HDR_FOUND(HDR__SUPERSEDES)) {
-      if (ARTidok(HDR(HDR__SUPERSEDES)))
+      if (IsValidMessageID(HDR(HDR__SUPERSEDES), true))
 	ARTcancel(data, HDR(HDR__SUPERSEDES), false);
     }
   }
