@@ -1,4 +1,4 @@
-/*  $Id: innd.c 8474 2009-05-17 13:12:15Z iulius $
+/*  $Id: innd.c 8886 2010-01-16 16:07:19Z iulius $
 **
 **  Variable definitions, miscellany, and main().
 */
@@ -203,10 +203,18 @@ const ARTHEADER ARTheaders[] = {
 /* #define HDR__X_ORIGINAL_NNTP_POSTING_HOST    68 */
   ARTHEADERINIT("Original-Sender",      HTstd),
 /* #define HDR__ORIGINAL_SENDER                 69 */
-  ARTHEADERINIT("NNTP-Posting-Path",    HTstd)
+  ARTHEADERINIT("NNTP-Posting-Path",    HTstd),
 /* #define HDR__NNTP_POSTING_PATH               70 */
+  ARTHEADERINIT("Archive",              HTstd),
+/* #define HDR__ARCHIVE                         71 */
+  ARTHEADERINIT("Archived-At",          HTstd),
+/* #define HDR__ARCHIVED_AT                     72 */
+  ARTHEADERINIT("Summary",              HTstd),
+/* #define HDR__SUMMARY                         73 */
+  ARTHEADERINIT("Comments",             HTstd)
+/* #define HDR__COMMENTS                        74 */
 };
-/* #define MAX_ARTHEADER                        71 */
+/* #define MAX_ARTHEADER                        75 */
 
 
 /*
@@ -422,7 +430,7 @@ main(int ac, char *av[])
 	    AnyIncoming = true;
 	    break;
 	case 'c':
-	    innconf->artcutoff = atoi(optarg);
+	    innconf->artcutoff = strtoul(optarg, NULL, 10);
 	    break;
  	case 'C':
  	    DoCancels = false;
@@ -437,10 +445,10 @@ main(int ac, char *av[])
 	    RemoteLimit = atoi(optarg);
 	    break;
 	case 'i':
-	    innconf->maxconnections = atoi(optarg);
+	    innconf->maxconnections = strtoul(optarg, NULL, 10);
 	    break;
 	case 'l':
-	    innconf->maxartsize = atol(optarg);
+	    innconf->maxartsize = strtoul(optarg, NULL, 10);
 	    break;
 	case 'm':
 	    if (ModeReason)
@@ -473,7 +481,7 @@ main(int ac, char *av[])
 	    MaxOutgoing = atoi(optarg);
 	    break;
 	case 'P':
-	    innconf->port = atoi(optarg);
+	    innconf->port = strtoul(optarg, NULL, 10);
 	    break;
 	case 'r':
 	    ShouldRenumber = true;
@@ -598,7 +606,7 @@ main(int ac, char *av[])
        Otherwise, we use insane amounts of memory for the channel table.
        FIXME: Get rid of this hard-coded constant. */
     if (i > 5000) {
-        int max;
+        unsigned long max;
 
         max = innconf->maxconnections + MaxOutgoing + 20;
         if (max < 5000)
@@ -650,7 +658,7 @@ main(int ac, char *av[])
     NCsetup();
     ARTsetup();
     ICDsetup(true);
-    if (innconf->timer)
+    if (innconf->timer != 0)
         TMRinit(TMR_MAX);
 
     /* Initialize the storage subsystem. */
