@@ -1,4 +1,4 @@
-/*  $Id: innd.h 8644 2009-10-01 19:24:43Z eagle $
+/*  $Id: innd.h 9009 2010-03-16 19:52:12Z iulius $
 **
 **  Many of the data types used here have abbreviations, such as CT for a
 **  channel type.  Here are a list of the conventions and meanings:
@@ -42,7 +42,7 @@
 #include "inn/messages.h"
 #include "inn/timer.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 #include "inn/storage.h"
 
@@ -116,24 +116,27 @@ typedef struct _ARTHEADER {
 
 
 /*
-**  Header content
+**  Header content.
 */
 typedef struct _HDRCONTENT {
-  char  *  Value;   /* don't copy, shows where it begins */
-  int	   Length;  /* Length of Value(tailing CRLF is not
-			   included.  -1 if duplicated */
+  char  *  Value;     /* Don't copy, shows where it begins. */
+  int      Length;    /* Length of Value(tailing CRLF is not
+                         included.  -1 if duplicated. */
+  char     LastChar;  /* Saved char when the last one is cut during parsing.
+                         Usually '\r' but it may be another char. */
 } HDRCONTENT;
 
 
 /*
 **  A way to index into the header table.
 */
-#define HDR_FOUND(_x)		(hc[(_x)].Length > 0)
-#define HDR_PARSE_START(_x)	hc[(_x)].Value[hc[_x].Length] = '\0'
-#define HDR(_x)			(hc[(_x)].Value)
+#define HDR_FOUND(_x)           (hc[(_x)].Length > 0)
+#define HDR_LASTCHAR_SAVE(_x)   hc[(_x)].LastChar = hc[(_x)].Value[hc[_x].Length]
+#define HDR_PARSE_START(_x)     hc[(_x)].Value[hc[_x].Length] = '\0'
+#define HDR(_x)                 (hc[(_x)].Value)
 /* HDR_LEN does not includes trailing "\r\n" */
-#define HDR_LEN(_x)		(hc[(_x)].Length)
-#define HDR_PARSE_END(_x)	hc[(_x)].Value[hc[_x].Length] = '\r'
+#define HDR_LEN(_x)             (hc[(_x)].Length)
+#define HDR_PARSE_END(_x)       hc[(_x)].Value[hc[_x].Length] = hc[(_x)].LastChar
 
 
 #define HDR__APPROVED		0
@@ -207,8 +210,12 @@ typedef struct _HDRCONTENT {
 #define HDR__X_ORIGINAL_NNTP_POSTING_HOST       68
 #define HDR__ORIGINAL_SENDER    69
 #define HDR__NNTP_POSTING_PATH  70
+#define HDR__ARCHIVE            71
+#define HDR__ARCHIVED_AT        72
+#define HDR__SUMMARY            73
+#define HDR__COMMENTS           74
 
-#define MAX_ARTHEADER           71
+#define MAX_ARTHEADER           75
 
 /*
 **  Miscellaneous data we want to keep on an article.  All the fields
@@ -314,12 +321,15 @@ enum channel_state {
 **  pointer to a channel, so set up a typedef for that.
 */
 struct _CHANNEL;
-typedef void (*innd_callback_func)(struct _CHANNEL *);
+typedef void (*innd_callback_func)     (struct _CHANNEL *);
+typedef void (*innd_callback_nntp_func)(struct _CHANNEL *);
 
 typedef struct _CHANNEL {
   enum channel_type    Type;
   enum channel_state   State;
   int		       fd;
+  int                  ac; /* Number of arguments in NNTP command. */
+  char              ** av; /* List of arguments in NNTP command. */
   bool		       Skip;
   bool                 Ignore;
   bool		       Streaming;
@@ -327,16 +337,22 @@ typedef struct _CHANNEL {
   bool		       privileged;
   bool		       Nolist;
   bool                 CanAuthenticate; /* Can use AUTHINFO? */
-  unsigned long	       Duplicate;
+  bool                 IsAuthenticated; /* No need to use AUTHINFO? */
+  bool                 HasSentUsername; /* Has used AUTHINFO USER? */
+  unsigned long        Duplicate;
+  unsigned long        Duplicate_checkpoint;
   unsigned long	       Unwanted_s;
   unsigned long	       Unwanted_f;
   unsigned long	       Unwanted_d;
   unsigned long	       Unwanted_g;
   unsigned long	       Unwanted_u;
   unsigned long	       Unwanted_o;
-  float		       Size;
-  float		       DuplicateSize;
-  float		       RejectSize;
+  float                Size;
+  float                Size_checkpoint;
+  float                DuplicateSize;
+  float                DuplicateSize_checkpoint;
+  float                RejectSize;
+  float                RejectSize_checkpoint;
   unsigned long	       Check;
   unsigned long	       Check_send;
   unsigned long	       Check_deferred;
@@ -350,21 +366,25 @@ typedef struct _CHANNEL {
   unsigned long	       Ihave_Deferred;
   unsigned long	       Ihave_SendIt;
   unsigned long	       Ihave_Cybercan;
-  int		       Reported;
-  long		       Received;
-  long		       Refused;
-  long		       Rejected;
-  int		       BadWrites;
-  int		       BadReads;
-  int		       BlockedWrites;
-  int		       BadCommands;
+  unsigned long        Reported;
+  unsigned long        Received;
+  unsigned long        Received_checkpoint;
+  unsigned long        Refused;
+  unsigned long        Refused_checkpoint;
+  unsigned long        Rejected;
+  unsigned long        Rejected_checkpoint;
+  unsigned long        BadWrites;
+  unsigned long        BadReads;
+  unsigned long        BlockedWrites;
+  unsigned int         BadCommands;
   time_t	       LastActive;
   time_t	       NextLog;
   struct sockaddr_storage Address;
   innd_callback_func   Reader;
   innd_callback_func   WriteDone;
   time_t	       Waketime;
-  time_t	       Started;
+  time_t               Started;
+  time_t               Started_checkpoint;
   innd_callback_func   Waker;
   void		    *  Argument;
   void		    *  Event;
@@ -474,6 +494,7 @@ typedef struct _SITE {
   bool		  NeedOverviewCreation;
   bool		  FeedwithoutOriginator;
   bool		  DropFiltered;
+  bool            FeedTrash;
   int		  Hops;
   int		  Groupcount;
   int		  Followcount;
@@ -622,7 +643,7 @@ EXTERN FILE	     *  Errlog;
 EXTERN FILE	     *  Log;
 extern char		LogName[];
 extern int		ErrorCount;
-EXTERN int		ICDactivedirty;
+EXTERN unsigned long    ICDactivedirty;
 EXTERN int		MaxOutgoing;
 EXTERN int		nGroups;
 EXTERN SITEIDX		nSites;
@@ -682,7 +703,6 @@ extern void		ThrottleNoMatchError(void);
 extern void		ReopenLog(FILE *F);
 extern void		xchown(char *p);
 
-extern bool		ARTidok(const char *MessageID);
 extern bool		ARTreadschema(void);
 extern const char   *	ARTreadarticle(char *files);
 extern char	    *   ARTreadheader(char *files);
@@ -744,8 +764,7 @@ extern void		ICDwriteactive(void);
 extern void		CCclose(void);
 extern void		CCsetup(void);
 
-extern void             KEYgenerate(HDRCONTENT *, const char *body, size_t,
-                                    const char *orig, size_t length);
+extern void             KEYgenerate(HDRCONTENT *, const char *body, size_t bodylen);
 
 extern void		LCclose(void);
 extern void		LCsetup(void);
