@@ -1,8 +1,8 @@
-/*  $Id: tinyleaf.c 9767 2014-12-07 21:13:43Z iulius $
+/*  $Id: tinyleaf.c 8838 2009-12-03 21:11:00Z iulius $
 **
 **  An extremely lightweight receive-only NNTP server.
 **
-**  Copyright 2003, 2004 Russ Allbery <eagle@eyrie.org>
+**  Copyright 2003, 2004 Russ Allbery <rra@stanford.edu>
 **
 **  Permission is hereby granted, free of charge, to any person obtaining a
 **  copy of this software and associated documentation files (the "Software"),
@@ -25,10 +25,10 @@
 
 #include "config.h"
 #include "clibrary.h"
+#include "portable/wait.h"
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <sys/wait.h>
 
 #include "inn/dispatch.h"
 #include "inn/messages.h"
