@@ -1,4 +1,4 @@
-/*  $Id: endpoint.c 8243 2008-12-21 11:50:47Z iulius $
+/*  $Id: endpoint.c 8781 2009-11-15 09:24:59Z iulius $
 **
 **  The implementation of the innfeed EndPoint object class.
 **
@@ -633,11 +633,11 @@ void Run (void)
       TMRstop(TMR_IDLE);
 
       timePasses () ;
-      if (innconf->timer)
+      if (innconf->timer != 0)
         {
 	  unsigned long now = TMRnow () ;
 	  if (last_summary == 0 
-	      || (long) (now - last_summary) > (innconf->timer * 1000))
+	      || (now - last_summary) > (innconf->timer * 1000))
 	    {
 	      TMRsummary ("ME", timer_name) ;
 	      last_summary = now;
