@@ -1,75 +1,35 @@
-/* -*- c -*-
- *
- * Author:      James Brister <brister@vix.com> -- berkeley-unix --
- * Start Date:  Mon, 15 Jan 1996 17:31:58 +1100
- * Project:     INN -- innfeed
- * File:        main.c
- * RCSId:       $Id: main.c,v 1.25.2.1 2000/06/30 15:15:36 kondou Exp $
- *
- * Copyright:   Copyright (c) 1996 by Internet Software Consortium
- *
- *              Permission to use, copy, modify, and distribute this
- *              software for any purpose with or without fee is hereby
- *              granted, provided that the above copyright notice and this
- *              permission notice appear in all copies.
- *
- *              THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE
- *              CONSORTIUM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- *              SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- *              MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET
- *              SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- *              INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- *              WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- *              WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- *              TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
- *              USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Description: Main routines for the innfeed program.
- * 
- */
-
-#if ! defined (lint)
-static const char *rcsid = "$Id: main.c,v 1.25.2.1 2000/06/30 15:15:36 kondou Exp $" ;
-static void use_rcsid (const char *rid) {   /* Never called */
-  use_rcsid (rcsid) ; use_rcsid (rid) ;
-}
-#endif
+/*  $Id: main.c 8781 2009-11-15 09:24:59Z iulius $
+**
+**  Main routines for the innfeed program.
+**
+**  Written by James Brister <brister@vix.com>
+*/
 
 #include "innfeed.h"
 #include "config.h"
 #include "clibrary.h"
+#include "portable/socket.h"
+#include "portable/time.h"
 
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <math.h>
 #include <netdb.h>
-#include <netinet/in.h>
 #include <signal.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <syslog.h>
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>
-#endif
-
-#ifdef HAVE_RLIMIT
-# include <sys/resource.h>
-#endif
-
-#ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#else
-# include <time.h>
-#endif
-
 #if defined(HAVE_UNIX_DOMAIN_SOCKETS)
 # include <sys/un.h>
 #endif
 
-#include "libinn.h"
-#include "storage.h"
+#include "inn/innconf.h"
+#include "inn/messages.h"
+#include "inn/version.h"
+#include "inn/libinn.h"
+#include "inn/storage.h"
 
 #include "article.h"
 #include "buffer.h"
@@ -79,7 +39,6 @@ static void use_rcsid (const char *rid) {   /* Never called */
 #include "host.h"
 #include "innlistener.h"
 #include "misc.h"
-#include "msgs.h"
 #include "tape.h"
 
 #define INHERIT 1
@@ -90,15 +49,21 @@ bool talkToSelf ;
 extern int debugWrites ;
 bool sigFlag = false ;
 const char *InputFile ;
-const char *configFile = NULL ;
+char *configFile = NULL ;
 bool RollInputFile = false ;
-const char *pidFile = NULL ;
+char *pidFile = NULL ;
 bool useMMap = false ;
 void (*gPrintInfo) (void) ;
 char *dflTapeDir;
+/* these are used by imapfeed */
+char *deliver_username  = NULL;
+char *deliver_authname  = NULL;
+char *deliver_password  = NULL;
+char *deliver_realm     = NULL;
+const char *deliver_rcpt_to   = "+%s";
+char *deliver_to_header = NULL;
 
 /* imports */
-extern char *versionInfo ;
 #if defined (sun)
 extern char *optarg ;           /* needed for Solaris */
 extern int optind;
@@ -134,7 +99,7 @@ static bool Mopt = false ;
 static bool Zopt = false ;
 static bool Dopt = false ;
 static int debugLevel = 0 ;
-static u_int initialSleep = 2 ;
+static unsigned int initialSleep = 2 ;
 static char *sopt = NULL ;
 static char *lopt = NULL ;
 static bool eopt = false ;
@@ -153,22 +118,20 @@ int main (int argc, char **argv)
   char *copt = NULL ;
   char *debugFile;
   bool checkConfig = false ;
-  struct rlimit rl;
   bool val;
 
+  timeToString (now, dateString, sizeof (dateString)) ;
 
-  strcpy (dateString,ctime(&now)) ;
-  dateString [24] = '\0' ;
-
-  if ((program = strrchr (argv [0],'/')) == NULL)
-    program = argv [0] ;
+  message_program_name = strrchr (argv [0],'/');
+  if (message_program_name == NULL)
+    message_program_name = argv [0] ;
   else
-    program++ ;
+    message_program_name++;
 
   gPrintInfo = gprintinfo ;
 
-  openlog (program,(int)(L_OPENLOG_FLAGS|LOG_PID),LOG_INN_PROG) ;
-  if (ReadInnConf() < 0) {
+  openlog (message_program_name,(int)(L_OPENLOG_FLAGS|LOG_PID),LOG_INN_PROG) ;
+  if (!innconf_read(NULL)) {
       syslog(LOG_ERR, "cant read inn.conf\n");
       exit(1);
   }
@@ -179,6 +142,10 @@ int main (int argc, char **argv)
   useMMap = false ;
 #endif
 
+  message_handlers_die (2, error_log_stderr_date, message_log_syslog_err) ;
+  message_handlers_warn (1, message_log_syslog_warning);
+  message_handlers_notice (1, message_log_syslog_notice) ;
+
 #define OPT_STRING "a:b:c:Cd:e:hl:mMo:p:S:s:vxyz"
 
   while ((optVal = getopt (argc,argv,OPT_STRING)) != EOF)
@@ -282,7 +249,7 @@ int main (int argc, char **argv)
 
   if (seenV)
     {
-      warn ("%s version: %s\n",program, versionInfo) ;
+      printf ("%s version: %s\n", message_program_name, INN_VERSION_STRING);
       exit (0) ;
     }
 
@@ -322,10 +289,10 @@ int main (int argc, char **argv)
 
   if ( !checkConfig ) 
     {
-      syslog (LOG_NOTICE,STARTING_PROGRAM,versionInfo,dateString) ;
+      notice ("ME starting at %s\n (%s)", dateString, INN_VERSION_STRING);
     }
 
-  val = TRUE;
+  val = true;
   if (!SMsetup(SM_PREOPEN, (void *)&val)) {
       syslog(LOG_ERR, "cant setup the storage subsystem\n");
       exit(1);
@@ -341,7 +308,7 @@ int main (int argc, char **argv)
       struct stat buf ;
 
       if (fstat (0,&buf) < 0)
-        logAndExit (1,FSTAT_FAILURE,"stdin") ;
+        logAndExit (1,"ME oserr fstat stdin: %s", strerror (errno)) ;
       else if (S_ISREG (buf.st_mode))
         InputFile = "";
     }
@@ -364,8 +331,8 @@ int main (int argc, char **argv)
                   "Empty pathname for ``-c'' option") ;
       exit (1) ;
     }
-  configFile = buildFilename(innconf->pathetc, copt ? copt : CONFIG_FILE);
-  dflTapeDir = buildFilename(innconf->pathspool, TAPE_DIRECTORY);
+  configFile = concatpath(innconf->pathetc, copt ? copt : CONFIG_FILE);
+  dflTapeDir = concatpath(innconf->pathspool, TAPE_DIRECTORY);
 
   rval = readConfig (configFile,(checkConfig ? stderr : NULL),
                      checkConfig,loggingLevel > 0);
@@ -380,20 +347,25 @@ int main (int argc, char **argv)
   if (checkConfig)
     {
       if (!rval)
-        fprintf (stderr,"config loading failed.\n") ;
+        {
+          fprintf (stderr,"config loading failed.\n") ;
+          exit (1) ;
+        }
       else
-        fprintf (stderr,"config loading succeeded.\n") ;
-      exit (1) ;
+        {
+          fprintf (stderr,"config loading succeeded.\n") ;
+          exit (0) ;
+        }
     }
   else if (!rval)
     exit (1) ;
 
-  debugFile = buildFilename (innconf->pathlog, DEBUG_FILE);
+  debugFile = concatpath(innconf->pathlog, DEBUG_FILE);
   if (loggingLevel == 0 && fileExistsP (debugFile))
     loggingLevel = 1 ;
 
   if (logFile == NULL && ! isatty (fileno (stderr)))
-    logFile = buildFilename (innconf->pathlog, LOG_FILE) ;
+    logFile = concatpath(innconf->pathlog, LOG_FILE);
 
   if (logFile)
     openLogFile () ;
@@ -408,15 +380,11 @@ int main (int argc, char **argv)
       int pid ;
 
       if (pipe (fds) < 0)
-        {
-          syslog (LOG_CRIT,PIPE_FAILURE) ;
-          exit (1) ;
-        }
+        sysdie ("ME fatal pipe") ;
 
       if ((pid = fork ()) < 0)
         {
-          syslog (LOG_CRIT,FORK_FAILURE) ;
-          exit (1) ;
+          sysdie ("ME fatal fork") ;
         }
       else if (pid == 0)
         {                       /* child */
@@ -426,7 +394,7 @@ int main (int argc, char **argv)
           close (2) ;
           dup2 (fds[1],1) ;
           dup2 (fds[1],2) ;
-          execlp ("sh","sh", "-c", subProgram, NULL) ;
+          execlp ("sh", "sh", "-c", subProgram, (char *) 0) ;
           perror ("execlp") ;
           exit (1) ;
         }
@@ -447,10 +415,7 @@ int main (int argc, char **argv)
       int pipefds [2] ;
 
       if (pipe (pipefds) != 0)
-        {
-          syslog (LOG_ERR,PIPE_FAILURE) ;
-          exit (1) ;
-        }
+        sysdie ("ME fatal pipe") ;
 
       close (0) ;
       dup2 (pipefds [0], 0) ;
@@ -460,10 +425,7 @@ int main (int argc, char **argv)
     }
 
   if (chdir (newsspool) != 0)
-    {
-      syslog (LOG_ERR,CD_FAILED,newsspool) ;
-      exit (1) ;
-    }
+    sysdie ("ME fatal chdir %s", newsspool) ;
 
     /* hook up the endpoint to the source of new article information (usually
        innd). */
@@ -478,33 +440,13 @@ int main (int argc, char **argv)
 
   sleep (initialSleep) ;
 
+  if (innconf->rlimitnofile >= 0)
+    if (setfdlimit (innconf->rlimitnofile) < 0)
+      syswarn ("ME oserr setrlimit(RLIM_NOFILE,%ld)", innconf->rlimitnofile) ;
 
-#if defined(HAVE_RLIMIT) && defined(RLIMIT_NOFILE)
-  /* now lower maximum open file limit to match what select(2) can handle. */
-  if (innconf->rlimitnofile >= 0) {
-    if (getrlimit(RLIMIT_NOFILE,&rl) != 0)
-      syslog (LOG_ERR,GETRLIM_FAILED) ;
-    else
-      {
-#if defined (FD_SETSIZE)
-	u_int fd_max = FD_SETSIZE ;
-#else
-	u_int fd_max = sizeof (fd_set) * CHAR_BIT ;
-#endif
-	if (innconf->rlimitnofile < fd_max)
-	  fd_max = innconf->rlimitnofile;
+  if (innconf->timer != 0)
+    TMRinit (TMR_MAX) ;
 
-	if (rl.rlim_max > fd_max)
-	  {
-	    rl.rlim_max = rl.rlim_cur = fd_max ;
-	    if (setrlimit (RLIMIT_NOFILE,&rl) != 0)
-	      syslog (LOG_ERR,SETRLIM_FAILED,(long)fd_max);
-	  }
-      }
-  }
-#endif /* HAVE_RLIMIT && RLIMIT_NOFILE */
-      
-  
   configHosts (talkToSelf) ;
 
   if (InputFile && *InputFile) {
@@ -541,8 +483,8 @@ int main (int argc, char **argv)
 static void usage (int val)
 {
   fprintf (stderr,"usage: %s [ options ] [ file ]\n\n",
-           program) ;
-  fprintf (stderr,"Version: %s\n\n",versionInfo) ;
+           message_program_name) ;
+  fprintf (stderr,"Version: %s\n\n",INN_VERSION_STRING) ;
   fprintf (stderr,"Config file: %s\n",CONFIG_FILE) ;
   fprintf (stderr,"Backlog directory: %s/%s\n", innconf->pathspool, TAPE_DIRECTORY) ;
   fprintf (stderr,"\nLegal options are:\n") ;
@@ -603,56 +545,43 @@ static void usage (int val)
   exit (val) ;
 }
 
-static void sigterm (int sig)
+static void sigterm (int sig UNUSED)
 {
-  (void) sig ;
-  
-  syslog(LOG_NOTICE, SHUTDOWN_SIGNAL);
+  notice ("ME received shutdown signal") ;
   shutDown (mainListener) ;
 }
 
-static void sigquit (int sig)
+static void sigquit (int sig UNUSED)
 {
-  (void) sig ;
-  
   sigterm (0) ;
 }
 
-static void sigint (int sig)
+static void sigint (int sig UNUSED)
 {
-  (void) sig ;
-
   gprintinfo () ;
 }
 
-static void sighup (int sig)
+static void sighup (int sig UNUSED)
 {
-  (void) sig ;
-  
-  syslog(LOG_NOTICE, CONFIG_RELOAD, configFile);
+  notice ("ME reloading config file %s", configFile) ;
 
   if (!readConfig (configFile,NULL,false,loggingLevel > 0))
     {
-      syslog (LOG_ERR,PARSE_FAILURE) ;
-      exit (1) ;
+      die ("ME config aborting, error parsing config file") ;
     }
 
   configHosts (talkToSelf) ;
 }
 
-static void sigemt (int sig)
+static void sigemt (int sig UNUSED)
 {
-  (void) sig ;
-
   gFlushTapes () ;
 }
 
-static void sigalrm (int sig)
+static void sigalrm (int sig UNUSED)
 {
-  (void) sig ;
-  
   if (InputFile == NULL)
-    syslog (LOG_ERR,IGNORE_SIGALRM) ;
+    warn ("ME signal SIGALRM in non-funnel-file mode ignored") ;
   else 
     {
       RollInputFile = true;
@@ -660,11 +589,8 @@ static void sigalrm (int sig)
     }
 }
 
-static void sigchld (int sig)
+static void sigchld (int sig UNUSED)
 {
-
-  (void) sig ;                  /* keep lint happy */
-
 #if 0
   wait (&status) ;              /* we don't care */
 #endif
@@ -676,11 +602,11 @@ static void sigchld (int sig)
 static void sigusr (int sig)
 {
   if (sig == SIGUSR1) {
-    syslog(LOG_NOTICE, INCR_LOGLEVEL, loggingLevel);
     loggingLevel++ ;
+    notice ("ME increasing logging level to %d", loggingLevel) ;
   } else if (sig == SIGUSR2 && loggingLevel > 0) {
-    syslog(LOG_NOTICE, DECR_LOGLEVEL, loggingLevel);
     loggingLevel-- ;
+    notice ("ME decreasing logging level to %d", loggingLevel) ;
   }    
 }
 
@@ -745,13 +671,14 @@ static void gprintinfo (void)
 {
   char *snapshotFile;
   FILE *fp;
+  char nowString[30];
   time_t now = theTime() ;
 
-  snapshotFile = buildFilename(innconf->pathlog, SNAPSHOT_FILE);
+  snapshotFile = concatpath(innconf->pathlog, SNAPSHOT_FILE);
   fp = fopen (snapshotFile,"a") ;
   if (fp == NULL)
     {
-      syslog (LOG_ERR,FOPEN_FAILURE,snapshotFile) ;
+      syswarn ("ME fopen %s", snapshotFile) ;
       free(snapshotFile);
       return ;
     }
@@ -763,8 +690,9 @@ static void gprintinfo (void)
   setbuf (fp, NULL) ;
 #endif
 
-  fprintf (fp,"----------------------------System snaphot taken at: %s\n",
-           ctime (&now)) ;
+  timeToString (now, nowString, sizeof (nowString)) ;
+  fprintf (fp,"----------------------------System snaphot taken at: %s\n\n",
+           nowString) ;
   gPrintListenerInfo (fp,0) ;
   fprintf (fp,"\n\n\n\n") ;
   gPrintHostInfo (fp,0) ;
@@ -791,16 +719,20 @@ static int mainConfigLoadCbk (void *data)
     {
       if ( !isDirectory (p) && isDirectory (innconf->patharticles) )
         {
-          logOrPrint (LOG_WARNING,fp,BADSPOOL_CHANGE,p,innconf->patharticles) ;
-          p = strdup (innconf->patharticles) ;
+          logOrPrint (LOG_WARNING,fp,
+                      "ME config: definition of news-spool (%s) is a"
+                      " non-existant directory. Using %s",p,
+                      innconf->patharticles) ;
+          p = xstrdup (innconf->patharticles) ;
         }
       else if (!isDirectory (p))
         logAndExit (1,"Bad spool directories: %s, %s\n",p,innconf->patharticles) ;
     }
   else if (!isDirectory (innconf->patharticles))
-    logAndExit (1,SPOOL_NODEF,innconf->patharticles);
+    logAndExit (1,"ME config: no definition of news-spool, and %s is no good",
+                innconf->patharticles);
   else
-    p = strdup (innconf->patharticles) ;
+    p = xstrdup (innconf->patharticles) ;
   newsspool = p ;
 
   /***************************************************/
@@ -808,7 +740,7 @@ static int mainConfigLoadCbk (void *data)
   if (getString (topScope,"input-file",&p,NO_INHERIT))
     {
       if (*p != '\0')
-	InputFile = buildFilename (getTapeDirectory(),p) ;
+	InputFile = concatpath(getTapeDirectory(), p);
       else
 	InputFile = "" ;
       free (p) ;
@@ -816,18 +748,18 @@ static int mainConfigLoadCbk (void *data)
   
   if (getString (topScope,"pid-file",&p,NO_INHERIT))
     {
-      pidFile = buildFilename (innconf->pathrun,p) ;
+      pidFile = concatpath(innconf->pathrun, p);
       free (p) ;
     }
   else
-    pidFile = buildFilename (innconf->pathrun,PID_FILE) ;
+    pidFile = concatpath(innconf->pathrun, PID_FILE);
   
   if (getInteger (topScope,"debug-level",&ival,NO_INHERIT))
-    loggingLevel = (u_int) ival ;
+    loggingLevel = (unsigned int) ival ;
   
   
   if (getInteger (topScope,"initial-sleep",&ival,NO_INHERIT))
-    initialSleep = (u_int) ival ;
+    initialSleep = (unsigned int) ival ;
   
   
   if (getBool (topScope,"use-mmap",&bval,NO_INHERIT))
@@ -836,9 +768,53 @@ static int mainConfigLoadCbk (void *data)
   
   if (getString (topScope,"log-file",&p,NO_INHERIT))
     {
-      logFile = buildFilename (innconf->pathlog,p) ;
-      FREE (p) ;
+      logFile = concatpath(innconf->pathlog, p);
+      free (p) ;
     }
+
+  if (getString (topScope,"log-time-format",&p,NO_INHERIT))
+    {
+      free(timeToStringFormat);
+      timeToStringFormat = p;
+    }
+
+   /* For imap/lmtp delivering */
+  if (getString (topScope,"deliver-username",&p, NO_INHERIT))
+    {   
+        deliver_username = p;
+      /* don't need to free */
+    }
+
+  if (getString (topScope,"deliver-authname",&p, NO_INHERIT))
+    {
+      deliver_authname = p;
+      /* don't need to free */
+    }
+
+  if (getString (topScope,"deliver-password",&p, NO_INHERIT))
+    {
+      deliver_password = p;
+      /* don't need to free */
+    }
+
+  if (getString (topScope,"deliver-realm",&p, NO_INHERIT))
+    {
+      deliver_realm = p;
+      /* don't need to free */
+    }
+
+  if (getString (topScope,"deliver-rcpt-to",&p, NO_INHERIT))
+    {
+      deliver_rcpt_to = p;
+      /* don't need to free */
+    }
+
+  if (getString (topScope,"deliver-to-header",&p, NO_INHERIT))
+    {
+      deliver_to_header = p;
+      /* don't need to free */
+    }
+
   
 
   return 1 ;
@@ -849,43 +825,41 @@ static int mainConfigLoadCbk (void *data)
  * can adjust config file values from options. They will be validated in the
  * second callback.
  */
-static int mainOptionsProcess (void *data)
+static int mainOptionsProcess (void *data UNUSED)
 {
   value *v ;
 
-  (void) data ;
-  
   if (bopt != NULL)
     {
       if ((v = findValue (topScope,"backlog-directory",NO_INHERIT)) != NULL) 
         {
-          FREE (v->v.charp_val) ;
-          v->v.charp_val = strdup (bopt) ;
+          free (v->v.charp_val) ;
+          v->v.charp_val = xstrdup (bopt) ;
         }
       else
-        addString (topScope,"backlog-directory",strdup (bopt)) ;
+        addString (topScope,"backlog-directory",xstrdup (bopt)) ;
     }
 
   if (aopt != NULL)
     {
       if ((v = findValue (topScope,"news-spool",NO_INHERIT)) != NULL)
         {
-          FREE (v->v.charp_val) ;
-          v->v.charp_val = strdup (aopt) ;
+          free (v->v.charp_val) ;
+          v->v.charp_val = xstrdup (aopt) ;
         }
       else
-        addString (topScope,"news-spool",strdup (aopt)) ;
+        addString (topScope,"news-spool",xstrdup (aopt)) ;
     }
 
   if (sopt != NULL)
     {
       if ((v = findValue (topScope,"status-file",NO_INHERIT)) != NULL)
         {
-          FREE (v->v.charp_val) ;
-          v->v.charp_val = strdup (sopt) ;
+          free (v->v.charp_val) ;
+          v->v.charp_val = xstrdup (sopt) ;
         }
       else
-        addString (topScope,"status-file",strdup (sopt)) ;
+        addString (topScope,"status-file",xstrdup (sopt)) ;
     }
 
 
@@ -923,11 +897,11 @@ static int mainOptionsProcess (void *data)
     {
       if ((v = findValue (topScope,"pid-file",NO_INHERIT)) != NULL)
         {
-          FREE (v->v.charp_val) ;
-          v->v.charp_val = strdup (popt) ;
+          free (v->v.charp_val) ;
+          v->v.charp_val = xstrdup (popt) ;
         }
       else
-        addString (topScope,"pid-file",strdup (popt)) ;
+        addString (topScope,"pid-file",xstrdup (popt)) ;
     }
 
   if (Zopt)
@@ -942,22 +916,22 @@ static int mainOptionsProcess (void *data)
     {
       if ((v = findValue (topScope,"log-file",NO_INHERIT)) != NULL)
         {
-          FREE (v->v.charp_val) ;
-          v->v.charp_val = strdup (lopt) ;
+          free (v->v.charp_val) ;
+          v->v.charp_val = xstrdup (lopt) ;
         }
       else
-        addString (topScope,"log-file",strdup (lopt)) ;
+        addString (topScope,"log-file",xstrdup (lopt)) ;
     }
 
   if (InputFile != NULL)
     {
       if ((v = findValue (topScope,"input-file",NO_INHERIT)) != NULL)
         {
-          FREE (v->v.charp_val) ;
-          v->v.charp_val = strdup (InputFile) ;
+          free (v->v.charp_val) ;
+          v->v.charp_val = xstrdup (InputFile) ;
         }
       else
-        addString (topScope,"input-file",strdup (InputFile)) ;
+        addString (topScope,"input-file",xstrdup (InputFile)) ;
     }
 
   return 1 ;
@@ -967,17 +941,21 @@ static int mainOptionsProcess (void *data)
 
 static void mainCleanup (void)
 {
-  FREE (configFile) ;
-  FREE (pidFile) ;
-  FREE (logFile) ;
-  FREE (newsspool) ;
+  free ((void *)configFile) ;
+  free ((void *)pidFile) ;
+  free (logFile) ;
+  free (newsspool) ;
+  configFile = NULL ;
+  pidFile = NULL ;
+  logFile = NULL ;
+  newsspool = NULL ;
 }
 
 
 void mainLogStatus (FILE *fp)
 {
   fprintf (fp,"%sGlobal configuration parameters:%s\n",
-           genHtml ? "<H2>" : "", genHtml ? "</H2>" : "") ;
+           genHtml ? "<B>" : "", genHtml ? "</B>" : "") ;
   fprintf (fp,"          Mode: ") ;
   if (InputFile != NULL)
     fprintf (fp,"Funnel file") ;
