@@ -1,4 +1,4 @@
-/*  $Id: defdist.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: defdist.c 8795 2009-11-15 09:28:29Z iulius $
 **
 */
 
@@ -55,7 +55,7 @@ DDstart(FILE *FromServer, FILE *ToServer)
             return NULL;
         close(fd);
 	if ((F = CA_listopen(name, FromServer, ToServer,
-		    "distrib.pats")) == NULL)
+		    "DISTRIB.PATS")) == NULL)
 	    return NULL;
     }
 
