@@ -1,4 +1,4 @@
-/*  $Id: cache.c 8014 2008-09-07 11:54:42Z iulius $
+/*  $Id: cache.c 8781 2009-11-15 09:24:59Z iulius $
 **
 **  Message-ID to storage token cache.
 **
@@ -31,7 +31,7 @@ static struct tst *msgidcache;
 **  (X)OVER, GROUP, (X)OVER, etc. for example doesn't blow up with
 **  out of memory.
 */
-static int msgcachecount;
+static unsigned long msgcachecount;
 
 struct cache_entry {
     struct node node;
