@@ -1,4 +1,4 @@
-/* $Id: api-t.c 10308 2018-12-02 14:33:59Z iulius $ */
+/* $Id: api-t.c 8964 2010-02-08 20:55:59Z iulius $ */
 /* Test suite for overview API. */
 
 #include "config.h"
@@ -13,7 +13,7 @@
 #include "inn/overview.h"
 #include "inn/vector.h"
 #include "inn/libinn.h"
-#include "tap/basic.h"
+#include "libtest.h"
 #include "inn/storage.h"
 
 /* Used as the artificial token for all articles inserted into overview. */
@@ -74,7 +74,6 @@ fake_innconf(void)
         free(innconf);
     }
     innconf = xmalloc(sizeof(*innconf));
-    memset(innconf, 0, sizeof(*innconf));
     innconf->enableoverview = true;
     innconf->groupbaseexpiry = true;
     innconf->icdsynccount = 10;
@@ -109,8 +108,7 @@ overview_init_buffindexed(void)
 static struct overview *
 overview_init(void)
 {
-    if (system("/bin/rm -rf ov-tmp") < 0)
-        sysdie("Cannot rm ov-tmp");
+    system("/bin/rm -rf ov-tmp");
     if (mkdir("ov-tmp", 0755))
         sysdie("Cannot mkdir ov-tmp");
     if (strcmp(innconf->ovmethod, "buffindexed") == 0)
@@ -141,7 +139,7 @@ overview_data_parse(char *data, unsigned long *artnum)
     *start = '\0';
     start++;
     *artnum = strtoul(start, NULL, 10);
-    if (*artnum == 0)
+    if (artnum == 0)
         die("Cannot parse article number in input data");
     return start;
 }
@@ -266,7 +264,7 @@ check_data(const char *group, ARTNUM artnum, const char *expected,
            struct overview_data *data)
 {
     bool status = true;
-    time_t expires UNUSED; // See below why it is still unused.
+    time_t expires;
     char *saw;
 
     if (artnum != data->number) {
@@ -296,7 +294,7 @@ check_data(const char *group, ARTNUM artnum, const char *expected,
     }
     if (data->arrived != (time_t) artnum * 10) {
         warn("Arrival time wrong for %s:%lu: %lu != %lu", group, artnum,
-             (unsigned long) data->arrived, artnum * 10);
+             data->arrived, artnum * 10);
         status = false;
     }
 
@@ -306,7 +304,7 @@ check_data(const char *group, ARTNUM artnum, const char *expected,
     expires = (artnum % 5 == 0) ? artnum * 100 : artnum;
     if (data->expires != 0) {
         warn("Expires time wrong for %s:%lu: %lu != %lu", group, artnum,
-             (unsigned long) data->expires, 0UL);
+             data->expires, 0UL);
         status = false;
     }
     return status;
@@ -394,10 +392,8 @@ overview_verify_search(const char *data, struct overview *overview)
     if (overdata == NULL)
         sysdie("Cannot open %s for reading", data);
     expected = vector_new();
-    if (fgets(buffer, sizeof(buffer), overdata) == NULL) {
-        fclose(overdata);
+    if (fgets(buffer, sizeof(buffer), overdata) == NULL)
         die("Unexpected end of file in %s", data);
-    }
     overview_data_parse(buffer, &artnum);
     group = xstrdup(buffer);
     while (fgets(buffer, sizeof(buffer), overdata) != NULL) {
@@ -415,7 +411,6 @@ overview_verify_search(const char *data, struct overview *overview)
         warn("Unable to open search for %s:%lu", buffer, start);
         free(group);
         vector_free(expected);
-        fclose(overdata);
         return false;
     }
     i = 0;
@@ -436,7 +431,6 @@ overview_verify_search(const char *data, struct overview *overview)
     }
     free(group);
     vector_free(expected);
-    fclose(overdata);
     return status;
 }
 
@@ -475,7 +469,6 @@ overview_verify_full_search(const char *data, struct overview *overview)
         warn("Unable to open full search for %s", group);
         free(group);
         vector_free(expected);
-        fclose(overdata);
         return false;
     }
     i = 0;
@@ -496,7 +489,6 @@ overview_verify_full_search(const char *data, struct overview *overview)
     }
     free(group);
     vector_free(expected);
-    fclose(overdata);
     return status;
 }
 
@@ -584,8 +576,7 @@ overview_tests(int n)
     ok(n++, overview_verify_data("overview/bogus", overview));
     hash_free(groups);
     overview_close(overview);
-    if (system("/bin/rm -rf ov-tmp") < 0)
-        sysdie("Cannot rm ov-tmp");
+    system("/bin/rm -rf ov-tmp");
     ok(n++, true);
     return n;
 }
@@ -681,8 +672,7 @@ overview_mmap_tests(int n)
     ok(n++, overview_verify_data("overview/bogus", overview));
     hash_free(groups);
     overview_close(overview);
-    if (system("/bin/rm -rf ov-tmp") < 0)
-        sysdie("Cannot rm ov-tmp");
+    system("/bin/rm -rf ov-tmp");
     ok(n++, true);
     return n;
 }
@@ -692,19 +682,12 @@ main(void)
 {
     int n = 1;
 
-    if (access("../data/overview/basic", F_OK) == 0) {
-        if (chdir("../data") < 0) {
-            sysbail("cannot chdir to ../data");
-        }
-    } else if (access("data/overview/basic", F_OK) == 0) {
-        if (chdir("data") < 0) {
-            sysbail("cannot chdir to data");
-        }
-    } else if (access("tests/data/overview/basic", F_OK) == 0) {
-        if (chdir("tests/data") < 0) {
-            sysbail("cannot chdir to tests/data");
-        }
-    }
+    if (access("../data/overview/basic", F_OK) == 0)
+        chdir("../data");
+    else if (access("data/overview/basic", F_OK) == 0)
+        chdir("data");
+    else if (access("tests/data/overview/basic", F_OK) == 0)
+        chdir("tests/data");
 
     /* Cancels can't be tested with mmap, so there are only 21 tests there. */
     test_init(27 * 2 + 21);
@@ -712,16 +695,16 @@ main(void)
     fake_innconf();
     innconf->ovmethod = xstrdup("tradindexed");
     innconf->tradindexedmmap = true;
-    diag("tradindexed with mmap");
+    printf("# tradindexed with mmap\n");
     n = overview_tests(1);
 
-    diag("tradindexed without mmap");
+    printf("# tradindexed without mmap\n");
     n = overview_mmap_tests(n);
 
     free(innconf->ovmethod);
     innconf->ovmethod = xstrdup("buffindexed");
-    diag("buffindexed");
-    overview_tests(n);
+    printf("# buffindexed\n");
+    n = overview_tests(n);
 
     return 0;
 }
